@@ -380,6 +380,16 @@ gtk_source_buffer_class_init (GtkSourceBufferClass *klass)
 	param_types[0] = GTK_TYPE_TEXT_ITER | G_SIGNAL_TYPE_STATIC_SCOPE;
 	param_types[1] = GTK_TYPE_TEXT_ITER | G_SIGNAL_TYPE_STATIC_SCOPE;
 
+	/**
+	 * GtkSourceBuffer::highlight-updated:
+	 * @buffer: the buffer that received the signal
+	 * @start: the start of the updated region
+	 * @end: the end of the updated region
+	 *
+	 * The ::highlight-updated signal is emitted when the highlighting is
+	 * updated in a certain region of the @buffer. It can be the syntax
+	 * highlighting, the search highlighting, etc.
+	 */
 	buffer_signals[HIGHLIGHT_UPDATED] =
 	    g_signal_newv ("highlight_updated",
 			   G_OBJECT_CLASS_TYPE (object_class),
@@ -408,6 +418,13 @@ gtk_source_buffer_class_init (GtkSourceBufferClass *klass)
 			   G_TYPE_NONE,
 			   1, GTK_TYPE_TEXT_MARK);
 
+	/**
+	 * GtkSourceBuffer::undo:
+	 * @buffer: the buffer that received the signal
+	 *
+	 * The ::undo signal is emitted to undo the last user action which
+	 * modified the buffer.
+	 */
 	buffer_signals[UNDO] =
 	    g_signal_new ("undo",
 			  G_OBJECT_CLASS_TYPE (object_class),
@@ -418,6 +435,12 @@ gtk_source_buffer_class_init (GtkSourceBufferClass *klass)
 			  G_TYPE_NONE,
 			  0);
 
+	/**
+	 * GtkSourceBuffer::redo:
+	 * @buffer: the buffer that received the signal
+	 *
+	 * The ::redo signal is emitted to redo the last undo operation.
+	 */
 	buffer_signals[REDO] =
 	    g_signal_new ("redo",
 			  G_OBJECT_CLASS_TYPE (object_class),
@@ -1282,6 +1305,8 @@ gtk_source_buffer_can_redo (GtkSourceBuffer *buffer)
  * Undoes the last user action which modified the buffer.  Use
  * gtk_source_buffer_can_undo() to check whether a call to this
  * function will have any effect.
+ *
+ * This function emits the #GtkSourceBuffer::undo signal.
  **/
 void
 gtk_source_buffer_undo (GtkSourceBuffer *buffer)
@@ -1297,6 +1322,8 @@ gtk_source_buffer_undo (GtkSourceBuffer *buffer)
  *
  * Redoes the last undo operation.  Use gtk_source_buffer_can_redo()
  * to check whether a call to this function will have any effect.
+ *
+ * This function emits the #GtkSourceBuffer::redo signal.
  **/
 void
 gtk_source_buffer_redo (GtkSourceBuffer *buffer)
@@ -2004,7 +2031,7 @@ gtk_source_buffer_get_source_marks_at_iter (GtkSourceBuffer *buffer,
  * @category: (allow-none): category to search for, or %NULL
  *
  * Returns the list of marks of the given category at @line.
- * If @category is NULL, all marks at @line are returned.
+ * If @category is %NULL, all marks at @line are returned.
  *
  * Returns: (element-type GtkSource.Mark) (transfer container):
  * a newly allocated #GSList.
@@ -2268,6 +2295,142 @@ gtk_source_buffer_iter_backward_to_context_class_toggle (GtkSourceBuffer *buffer
 	}
 }
 
+static void
+do_lower_case (GtkTextBuffer *buffer,
+               GtkTextIter   *start,
+               GtkTextIter   *end,
+               GString       *str)
+{
+	while (!gtk_text_iter_is_end (start) &&
+	       !gtk_text_iter_equal (start, end))
+	{
+		gunichar c, nc;
+
+		c = gtk_text_iter_get_char (start);
+		nc = g_unichar_tolower (c);
+		g_string_append_unichar (str, nc);
+
+		gtk_text_iter_forward_char (start);
+	}
+}
+
+static void
+do_upper_case (GtkTextBuffer *buffer,
+               GtkTextIter   *start,
+               GtkTextIter   *end,
+               GString       *str)
+{
+	while (!gtk_text_iter_is_end (start) &&
+	       !gtk_text_iter_equal (start, end))
+	{
+		gunichar c, nc;
+
+		c = gtk_text_iter_get_char (start);
+		nc = g_unichar_toupper (c);
+		g_string_append_unichar (str, nc);
+
+		gtk_text_iter_forward_char (start);
+	}
+}
+
+static void
+do_toggle_case (GtkTextBuffer *buffer,
+                GtkTextIter   *start,
+                GtkTextIter   *end,
+                GString       *str)
+{
+	while (!gtk_text_iter_is_end (start) &&
+	       !gtk_text_iter_equal (start, end))
+	{
+		gunichar c, nc;
+
+		c = gtk_text_iter_get_char (start);
+		if (g_unichar_islower (c))
+			nc = g_unichar_toupper (c);
+		else
+			nc = g_unichar_tolower (c);
+		g_string_append_unichar (str, nc);
+
+		gtk_text_iter_forward_char (start);
+	}
+}
+
+static void
+do_title_case (GtkTextBuffer *buffer,
+               GtkTextIter   *start,
+               GtkTextIter   *end,
+               GString       *str)
+{
+
+	while (!gtk_text_iter_is_end (start) &&
+	       !gtk_text_iter_equal (start, end))
+	{
+		gunichar c, nc;
+
+		c = gtk_text_iter_get_char (start);
+		if (gtk_text_iter_starts_word (start))
+			nc = g_unichar_totitle (c);
+		else
+			nc = g_unichar_tolower (c);
+		g_string_append_unichar (str, nc);
+
+		gtk_text_iter_forward_char (start);
+	}
+}
+
+/**
+ * gtk_source_buffer_change_case:
+ * @buffer: a #GtkSourceBuffer.
+ * @case_type: how to change the case.
+ * @start: a #GtkTextIter.
+ * @end: a #GtkTextIter.
+ *
+ * Changes the case of the text between the specified iterators.
+ *
+ * Since: 3.12
+ **/
+void
+gtk_source_buffer_change_case (GtkSourceBuffer         *buffer,
+                               GtkSourceChangeCaseType  case_type,
+                               GtkTextIter             *start,
+                               GtkTextIter             *end)
+{
+	GtkTextIter i;
+	GString *s;
+
+	g_return_if_fail (GTK_SOURCE_IS_BUFFER (buffer));
+	g_return_if_fail (start != NULL);
+	g_return_if_fail (end != NULL);
+
+	i = *start;
+	s = g_string_new (NULL);
+
+	switch (case_type)
+	{
+	case GTK_SOURCE_CHANGE_CASE_UPPER:
+		do_upper_case (GTK_TEXT_BUFFER (buffer), &i, end, s);
+		break;
+	case GTK_SOURCE_CHANGE_CASE_LOWER:
+		do_lower_case (GTK_TEXT_BUFFER (buffer), &i, end, s);
+		break;
+	case GTK_SOURCE_CHANGE_CASE_TOGGLE:
+		do_toggle_case (GTK_TEXT_BUFFER (buffer), &i, end, s);
+		break;
+	case GTK_SOURCE_CHANGE_CASE_TITLE:
+		do_title_case (GTK_TEXT_BUFFER (buffer), &i, end, s);
+		break;
+	default:
+		g_return_if_reached ();
+	}
+
+	gtk_text_buffer_begin_user_action (GTK_TEXT_BUFFER (buffer));
+	gtk_text_buffer_delete_interactive (GTK_TEXT_BUFFER (buffer), start, end, TRUE);
+	gtk_text_buffer_insert_interactive (GTK_TEXT_BUFFER (buffer), start, s->str, s->len, TRUE);
+	gtk_text_buffer_end_user_action (GTK_TEXT_BUFFER (buffer));
+
+	g_string_free (s, TRUE);
+}
+
 /**
  * gtk_source_buffer_set_undo_manager:
  * @buffer: a #GtkSourceBuffer.
